package clientutil

import (
	"errors"
	"net"
	"net/http"
	"time"

	"github.com/cenkalti/backoff"
)

func NewExponentialBackOff() *backoff.ExponentialBackOff {
	b := backoff.NewExponentialBackOff()
	b.InitialInterval = 100 * time.Millisecond
	//b.Multiplier = 1.4142
	return b
}

func Retry(op backoff.Operation, b backoff.BackOff) error {
	innerOp := func() error {
		err := op()
		if err == nil {
			return err
		}
		if netErr, ok := err.(net.Error); ok && netErr.Temporary() {
			return err
		}
		return backoff.Permanent(err)
	}
	return backoff.Retry(innerOp, b)
}

var errHTTPBackOff = errors.New("http status 503")

func RetryHTTPDo(client *http.Client, req *http.Request, b backoff.BackOff) (*http.Response, error) {
	var resp *http.Response
	op := func() error {
		// Clear up previous response if set.
		if resp != nil {
			resp.Body.Close()
		}

		var err error
		resp, err = client.Do(req)
		if err == nil && resp.StatusCode == 503 {
			resp.Body.Close()
			return errHTTPBackOff
		}
		return err
	}

	err := Retry(op, b)
	return resp, err
}
